#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Sun Nov  3 18:48:59 2019
@author: Olivier THUAL

What time is it?
Data assimilation with two clocks

"""

import numpy as np
import matplotlib.pyplot as plt

plt.close('all')


# Parameters
Tt=60 # True state
sig1=4 ; C1=1/sig1**2 # T1
sig2=5 ; C2=1/sig2**2 # T2
Ca=C1+C2; siga=np.sqrt(1/Ca) # Ta

def fT(T,Tt,sig):
    # Gaussian probability density function 
    K=1/(np.sqrt(2*np.pi)*sig)
    f = K*np.exp(-(T-Tt)**2/(2*sig**2))
    return f,K

def val(Tt,sig,col) :
    # Ploting the probability density functions
    T = np.linspace(Tt-3*sig, Tt+3*sig, 100)
    f,K=fT(T,Tt,sig)
    plt.plot(T, f, color=col, linewidth=3)
    # Random sampling
    Tr = np.random.normal(Tt,sig,20)
    fR,KR=fT(Tr,Tt,sig)
    plt.scatter(Tr,fR,marker='o',linewidth=3,color=col)
    plt.axhline(0)
    plt.axvline(Tt)
    plt.title('Probability density functions',fontsize=16)
    plt.xlabel('$T$',fontsize=32 )
    plt.xticks(fontsize=16)
    plt.yticks(fontsize=16)
    plt.ylabel('$f_{T}$',fontsize=32)
    
 

def histo(i) :
    # Ploting random draws on a line    
    plt.figure()
    val(Tt,sig1,'black');
    val(Tt,sig2,'green')
    val(Tt,siga,'red')
    # one draw
    T1=np.random.normal(Tt,sig1)
    T2=np.random.normal(Tt,sig2)
    Ta=(C1*T1+C2*T2)/Ca
    plt.scatter(Tt,0,marker='^',color='blue',s=200)
    plt.scatter(T1,0,marker='^',color='black',s=200)
    plt.scatter(T2,0,marker='^',color='green',s=200)
    plt.scatter(Ta,0,marker='^',color='red',s=200)

def improve(Nc):
    # Counting if T2 improves the analysis
    count=0
    for i in range(Nc) :
        T1=np.random.normal(Tt,sig1)
        T2=np.random.normal(Tt,sig2)
        Ta=(C1*T1+C2*T2)/Ca      
        if abs(Ta-Tt) < abs(T1-Tt) :
            count=count+1
    
    rho=count/Nc
    return rho

"""
The program starts here
"""    
     
      
# Plotting the histograms
if True :
    for i in range(9) :
        histo(i)
        #plt.draw()
        plt.savefig('hist'+str(i)+'.pdf')


# Counting the improvements
if True :
    rho=improve(1000)
    print('=================================')
    print('Rate of improvemennts rho=',rho)
    print('=================================')
    

# Scores for the estimation of sigma_a
if True :
    Rmax=1000; scores=[]
    
    for R in np.arange(2,Rmax+1) :
        ET=0; VT=0
        T1=np.random.normal(Tt,sig1,R)  
        T2=np.random.normal(Tt,sig2,R) 
        Ta=(C1*T1+C2*T2)/Ca
        E=np.sum(Ta)/R
        V=np.sum((Ta-E)**2)/(R-1)
        sigaR=np.sqrt(V)
        scor=(sigaR-siga)/siga
        scores=np.append(scores,scor)

    plt.figure()
    plt.axhline(0)
    plt.axvline(0)
    plt.xticks(fontsize=16)
    plt.yticks(fontsize=16)
    plt.plot(np.log10(np.abs(scores)),color='red')
    plt.title('Estimation of $\sigma_a$',fontsize=16)
    plt.xlabel('$R$',fontsize=24 )
    plt.ylabel('$\log_{10}(|Score|)$',fontsize=16)
    plt.savefig('scores.pdf')
    
# Show all figures in command line model
print("Close all figures to stop the program")
plt.show()
    
      




